"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadRules = exports.loadFilterRules = exports.loadPlugins = void 0;
const config_util_1 = require("./config-util");
const module_interop_1 = require("@textlint/module-interop");
const preset_loader_1 = require("./preset-loader");
const is_1 = require("./is");
const import_1 = require("./import");
const isPluginCreator = (mod) => {
    return typeof mod === "object" && Object.prototype.hasOwnProperty.call(mod, "Processor");
};
const loadPlugins = ({ pluginsObject, moduleResolver, testReplaceDefinitions }) => __awaiter(void 0, void 0, void 0, function* () {
    const plugins = [];
    const pluginErrors = [];
    if (Array.isArray(pluginsObject)) {
        // { plugins: ["a", "b"] }
        yield Promise.all(pluginsObject.map((pluginId) => __awaiter(void 0, void 0, void 0, function* () {
            const resolvedModule = moduleResolver.resolvePluginPackageName(pluginId);
            const mod = yield (0, import_1.dynamicImport)(resolvedModule.filePath);
            const plugin = (0, module_interop_1.moduleInterop)(mod.default);
            if (!isPluginCreator(plugin)) {
                pluginErrors.push(new Error(`Plugin should be object that has "Processor" property. But "${pluginId}" is not.

Please check "${pluginId}" is valid plugin.
FilePath: ${resolvedModule.filePath}

For more details, See FAQ: https://github.com/textlint/textlint/blob/master/docs/faq/failed-to-load-textlints-module.md
`));
            }
            plugins.push({
                pluginId,
                plugin,
                filePath: resolvedModule.filePath,
                moduleName: resolvedModule.moduleName
            });
        })));
    }
    else {
        // { plugins: { "a": true, "b": options } }
        yield Promise.all(Object.entries(pluginsObject).map(([pluginId, pluginOptions]) => __awaiter(void 0, void 0, void 0, function* () {
            try {
                // Test Replace logic
                const replacedDefinition = testReplaceDefinitions &&
                    testReplaceDefinitions.find((definition) => {
                        return definition.pluginId === pluginId;
                    });
                if (replacedDefinition) {
                    // for debug
                    plugins.push(replacedDefinition);
                }
                else {
                    const resolvedPlugin = moduleResolver.resolvePluginPackageName(pluginId);
                    const mod = yield (0, import_1.dynamicImport)(resolvedPlugin.filePath);
                    const plugin = (0, module_interop_1.moduleInterop)(mod.default);
                    if (!isPluginCreator(plugin)) {
                        pluginErrors.push(new Error(`Plugin should be object that has "Processor" property. But "${pluginId}" is not.

Please check "${pluginId}" is valid plugin.
FilePath: ${resolvedPlugin.filePath}

For more details, See FAQ: https://github.com/textlint/textlint/blob/master/docs/faq/failed-to-load-textlints-module.md
`));
                        return;
                    }
                    plugins.push({
                        pluginId,
                        plugin,
                        options: pluginOptions,
                        filePath: resolvedPlugin.filePath,
                        moduleName: resolvedPlugin.moduleName
                    });
                }
            }
            catch (error) {
                pluginErrors.push(error);
            }
        })));
    }
    return {
        plugins,
        pluginsError: pluginErrors.length === 0
            ? null
            : {
                message: "Can not load plugin",
                errors: pluginErrors
            }
    };
});
exports.loadPlugins = loadPlugins;
const loadFilterRules = ({ rulesObject, moduleResolver, testReplaceDefinitions }) => __awaiter(void 0, void 0, void 0, function* () {
    // rules
    const rules = [];
    const ruleErrors = [];
    yield Promise.all(Object.entries(rulesObject).map(([ruleId, ruleOptions]) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            // Test Replace logic
            const replacedDefinition = testReplaceDefinitions &&
                testReplaceDefinitions.find((definition) => {
                    return definition.ruleId === ruleId;
                });
            if (replacedDefinition) {
                // for debug
                rules.push(replacedDefinition);
            }
            else {
                const resolvePackage = moduleResolver.resolveFilterRulePackageName(ruleId);
                const mod = yield (0, import_1.dynamicImport)(resolvePackage.filePath);
                const ruleModule = (0, module_interop_1.moduleInterop)(mod.default);
                if (!(0, is_1.isTextlintFilterRuleReporter)(ruleModule)) {
                    ruleErrors.push(new Error(`Filter rule should be object that has "filter" property. But ${ruleId} is not.`));
                    return;
                }
                // rule
                rules.push({
                    ruleId,
                    rule: ruleModule,
                    options: ruleOptions,
                    filePath: resolvePackage.filePath,
                    moduleName: resolvePackage.moduleName
                });
            }
        }
        catch (error) {
            ruleErrors.push(error);
        }
    })));
    return {
        filterRules: rules,
        filterRulesError: ruleErrors.length === 0
            ? null
            : {
                message: "Can not load filter rule",
                errors: ruleErrors
            }
    };
});
exports.loadFilterRules = loadFilterRules;
const loadRules = ({ rulesObject, moduleResolver, testReplaceDefinitions }) => __awaiter(void 0, void 0, void 0, function* () {
    // rules
    const rules = [];
    const ruleErrors = [];
    yield Promise.all(Object.entries(rulesObject).map(([ruleId, ruleOptions]) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            // Test Replace logic
            const replacedDefinition = testReplaceDefinitions &&
                testReplaceDefinitions.find((definition) => {
                    return definition.ruleId === ruleId;
                });
            // if rule is disabled, skip to load
            if (!ruleOptions) {
                return;
            }
            if (replacedDefinition) {
                // for debug
                rules.push(replacedDefinition);
            }
            else {
                if ((0, config_util_1.isPresetRuleKey)(ruleId)) {
                    // load preset
                    const presetRulesOptions = typeof ruleOptions === "boolean" ? {} : ruleOptions;
                    const rulesInPreset = yield (0, preset_loader_1.loadPreset)({
                        presetName: ruleId,
                        presetRulesOptions,
                        moduleResolver
                    });
                    rules.push(...rulesInPreset);
                }
                else {
                    // load rule
                    const resolvePackage = moduleResolver.resolveRulePackageName(ruleId);
                    const mod = yield (0, import_1.dynamicImport)(resolvePackage.filePath);
                    const ruleModule = (0, module_interop_1.moduleInterop)(mod.default);
                    if (!(0, is_1.isTextlintRuleModule)(ruleModule)) {
                        ruleErrors.push(new Error(`Rule should have "rules" and "rulesConfig" properties. But ${ruleId} is not.
                        
Please check ${ruleId} is valid rule.
FilePath: ${resolvePackage.filePath}

For more details, See FAQ: https://github.com/textlint/textlint/blob/master/docs/faq/failed-to-load-textlints-module.md                        
`));
                        return;
                    }
                    // rule
                    rules.push({
                        ruleId,
                        rule: ruleModule,
                        options: ruleOptions,
                        filePath: resolvePackage.filePath,
                        moduleName: resolvePackage.moduleName
                    });
                }
            }
        }
        catch (error) {
            ruleErrors.push(error);
        }
    })));
    return {
        rules,
        rulesError: ruleErrors.length === 0
            ? null
            : {
                message: "Can not load rule",
                errors: ruleErrors
            }
    };
});
exports.loadRules = loadRules;
//# sourceMappingURL=loader.js.map